#pragma once

enum class Opcode : uint8_t
{
  RZP_ADC = 0x65,
  RZP_AND = 0x25,
  RZP_BIT = 0x24,
  RZP_CMP = 0xc5,
  RZP_CPX = 0xe4,
  RZP_CPY = 0xc4,
  RZP_EOR = 0x45,
  RZP_LDA = 0xa5,
  RZP_LDX = 0xa6,
  RZP_LDY = 0xa4,
  RZP_ORA = 0x05,
  RZP_SBC = 0xe5,

  WZP_STA = 0x85,
  WZP_STX = 0x86,
  WZP_STY = 0x84,
  WZP_STZ = 0x64,

  MZP_ASL = 0x06,
  MZP_DEC = 0xc6,
  MZP_INC = 0xe6,
  MZP_LSR = 0x46,
  MZP_RMB0 = 0x07,
  MZP_RMB1 = 0x17,
  MZP_RMB2 = 0x27,
  MZP_RMB3 = 0x37,
  MZP_RMB4 = 0x47,
  MZP_RMB5 = 0x57,
  MZP_RMB6 = 0x67,
  MZP_RMB7 = 0x77,
  MZP_ROL = 0x26,
  MZP_ROR = 0x66,
  MZP_SMB0 = 0x87,
  MZP_SMB1 = 0x97,
  MZP_SMB2 = 0xa7,
  MZP_SMB3 = 0xb7,
  MZP_SMB4 = 0xc7,
  MZP_SMB5 = 0xd7,
  MZP_SMB6 = 0xe7,
  MZP_SMB7 = 0xf7,
  MZP_TRB = 0x14,
  MZP_TSB = 0x04,

  RZX_ADC = 0x75,
  RZX_AND = 0x35,
  RZX_BIT = 0x34,
  RZX_CMP = 0xd5,
  RZX_EOR = 0x55,
  RZX_LDA = 0xb5,
  RZX_LDY = 0xb4,
  RZX_ORA = 0x15,
  RZX_SBC = 0xf5,

  RZY_LDX = 0xb6,

  WZX_STA = 0x95,
  WZX_STY = 0x94,
  WZX_STZ = 0x74,

  WZY_STX = 0x96,

  MZX_ASL = 0x16,
  MZX_DEC = 0xd6,
  MZX_INC = 0xf6,
  MZX_LSR = 0x56,
  MZX_ROL = 0x36,
  MZX_ROR = 0x76,

  RIN_ADC = 0x72,
  RIN_AND = 0x32,
  RIN_CMP = 0xd2,
  RIN_EOR = 0x52,
  RIN_LDA = 0xb2,
  RIN_ORA = 0x12,
  RIN_SBC = 0xf2,

  WIN_STA = 0x92,

  RIX_ADC = 0x61,
  RIX_AND = 0x21,
  RIX_CMP = 0xc1,
  RIX_EOR = 0x41,
  RIX_LDA = 0xa1,
  RIX_ORA = 0x01,
  RIX_SBC = 0xe1,

  WIX_STA = 0x81,

  RIY_ADC = 0x71,
  RIY_AND = 0x31,
  RIY_CMP = 0xd1,
  RIY_EOR = 0x51,
  RIY_LDA = 0xb1,
  RIY_ORA = 0x11,
  RIY_SBC = 0xf1,

  WIY_STA = 0x91,

  RAB_ADC = 0x6d,
  RAB_AND = 0x2d,
  RAB_BIT = 0x2c,
  RAB_CMP = 0xcd,
  RAB_CPX = 0xec,
  RAB_CPY = 0xcc,
  RAB_EOR = 0x4d,
  RAB_LDA = 0xad,
  RAB_LDX = 0xae,
  RAB_LDY = 0xac,
  RAB_ORA = 0x0d,
  RAB_SBC = 0xed,

  WAB_STA = 0x8d,
  WAB_STX = 0x8e,
  WAB_STY = 0x8c,
  WAB_STZ = 0x9c,

  MAB_ASL = 0x0e,
  MAB_DEC = 0xce,
  MAB_INC = 0xee,
  MAB_LSR = 0x4e,
  MAB_ROL = 0x2e,
  MAB_ROR = 0x6e,
  MAB_TRB = 0x1c,
  MAB_TSB = 0x0c,

  RAX_ADC = 0x7d,
  RAX_AND = 0x3d,
  RAX_BIT = 0x3c,
  RAX_CMP = 0xdd,
  RAX_EOR = 0x5d,
  RAX_LDA = 0xbd,
  RAX_LDY = 0xbc,
  RAX_ORA = 0x1d,
  RAX_SBC = 0xfd,
  RAY_ADC = 0x79,
  RAY_AND = 0x39,
  RAY_CMP = 0xd9,
  RAY_EOR = 0x59,
  RAY_LDA = 0xb9,
  RAY_LDX = 0xbe,
  RAY_ORA = 0x19,
  RAY_SBC = 0xf9,

  WAX_STA = 0x9d,
  WAX_STZ = 0x9e,

  WAY_STA = 0x99,

  MAX_ASL = 0x1e,
  MAX_DEC = 0xde,
  MAX_INC = 0xfe,
  MAX_LSR = 0x5e,
  MAX_ROL = 0x3e,
  MAX_ROR = 0x7e,

  JMA_JMP = 0x4c,

  JSA_JSR = 0x20,

  JMX_JMP = 0x7c,

  JMI_JMP = 0x6c,

  IMP_ASL = 0x0a,
  IMP_CLC = 0x18,
  IMP_CLD = 0xd8,
  IMP_CLI = 0x58,
  IMP_CLV = 0xb8,
  IMP_DEC = 0x3a,
  IMP_DEX = 0xca,
  IMP_DEY = 0x88,
  IMP_INC = 0x1a,
  IMP_INX = 0xe8,
  IMP_INY = 0xc8,
  IMP_LSR = 0x4a,
  IMP_NOP = 0xea,
  IMP_ROL = 0x2a,
  IMP_ROR = 0x6a,
  IMP_SEC = 0x38,
  IMP_SED = 0xf8,
  IMP_SEI = 0x78,
  IMP_TAX = 0xaa,
  IMP_TAY = 0xa8,
  IMP_TSX = 0xba,
  IMP_TXA = 0x8a,
  IMP_TXS = 0x9a,
  IMP_TYA = 0x98,

  IMM_ADC = 0x69,
  IMM_AND = 0x29,
  IMM_BIT = 0x89,
  IMM_CMP = 0xc9,
  IMM_CPX = 0xe0,
  IMM_CPY = 0xc0,
  IMM_EOR = 0x49,
  IMM_LDA = 0xa9,
  IMM_LDX = 0xa2,
  IMM_LDY = 0xa0,
  IMM_ORA = 0x09,
  IMM_SBC = 0xe9,

  BRL_BCC = 0x90,
  BRL_BCS = 0xb0,
  BRL_BEQ = 0xf0,
  BRL_BMI = 0x30,
  BRL_BNE = 0xd0,
  BRL_BPL = 0x10,
  BRL_BRA = 0x80,
  BRL_BVC = 0x50,
  BRL_BVS = 0x70,

  BZR_BBR0 = 0x0f,
  BZR_BBR1 = 0x1f,
  BZR_BBR2 = 0x2f,
  BZR_BBR3 = 0x3f,
  BZR_BBR4 = 0x4f,
  BZR_BBR5 = 0x5f,
  BZR_BBR6 = 0x6f,
  BZR_BBR7 = 0x7f,
  BZR_BBS0 = 0x8f,
  BZR_BBS1 = 0x9f,
  BZR_BBS2 = 0xaf,
  BZR_BBS3 = 0xbf,
  BZR_BBS4 = 0xcf,
  BZR_BBS5 = 0xdf,
  BZR_BBS6 = 0xef,
  BZR_BBS7 = 0xff,

  BRK_BRK = 0x00,

  RTI_RTI = 0x40,

  RTS_RTS = 0x60,

  PHR_PHA = 0x48,
  PHR_PHP = 0x08,
  PHR_PHX = 0xda,
  PHR_PHY = 0x5a,

  PLR_PLA = 0x68,
  PLR_PLP = 0x28,
  PLR_PLX = 0xfa,
  PLR_PLY = 0x7a,

  UND_1_03 = 0x03,
  UND_1_13 = 0x13,
  UND_1_23 = 0x23,
  UND_1_33 = 0x33,
  UND_1_43 = 0x43,
  UND_1_53 = 0x53,
  UND_1_63 = 0x63,
  UND_1_73 = 0x73,
  UND_1_83 = 0x83,
  UND_1_93 = 0x93,
  UND_1_a3 = 0xa3,
  UND_1_b3 = 0xb3,
  UND_1_c3 = 0xc3,
  UND_1_d3 = 0xd3,
  UND_1_e3 = 0xe3,
  UND_1_f3 = 0xf3,

  UND_1_0b = 0x0b,
  UND_1_1b = 0x1b,
  UND_1_2b = 0x2b,
  UND_1_3b = 0x3b,
  UND_1_4b = 0x4b,
  UND_1_5b = 0x5b,
  UND_1_6b = 0x6b,
  UND_1_7b = 0x7b,
  UND_1_8b = 0x8b,
  UND_1_9b = 0x9b,
  UND_1_ab = 0xab,
  UND_1_bb = 0xbb,
  UND_1_cb = 0xcb,
  UND_1_db = 0xdb,
  UND_1_eb = 0xeb,
  UND_1_fb = 0xfb,

  UND_2_02 = 0x02,
  UND_2_22 = 0x22,
  UND_2_42 = 0x42,
  UND_2_62 = 0x62,
  UND_2_82 = 0x82,
  UND_2_C2 = 0xC2,
  UND_2_E2 = 0xE2,

  UND_3_44 = 0x44,
  UND_4_54 = 0x54,
  UND_4_d4 = 0xd4,
  UND_4_f4 = 0xf4,
  UND_4_dc = 0xdc,
  UND_4_fc = 0xfc,
  UND_8_5c = 0x5c
};
